import femto.mode.HiRes16Color;
import femto.sound.Procedural;
import femto.Image;
import femto.Sprite;

import Common;

class Event
{
    // enum EventType
    // {
    //     NONE, START_BUBBLE, END_BUBBLE
    // }
    
    static final int EVENT_NONE = 0;
    static final int EVENT_START_BUBBLE = 1;
    static final int EVENT_END_BUBBLE = 2;
    static final int EVENT_SFX = 3;
    static final int EVENT_RANDOM_REPEATABLE_SFX = 4;
    static final int EVENT_START_ZOMBIE_SPIRAL = 5;
    static final int EVENT_END_ZOMBIE_SPIRAL = 6;
    
    // enum EventState
    // {
    //     PENDING, RUNNING
    // }
    
    Event() 
    {
        type = EVENT_NONE;
    }
    
    public void clear() 
    {
        eventTime = 0;
        type = EVENT_NONE;
        isDrawingState = false;
    }
    
    public void setBubbleTextEvent(long startTime, String textLinePar1, String textLinePar2, Bubble bubbleImagePar ) 
    {
        eventTime = Common.currentFrameStartTimeMs + startTime;
        type = EVENT_START_BUBBLE;
        isDrawingState = false;  
        image = bubbleImagePar;
        textLine1 = textLinePar1;
        textLine2 = textLinePar2;
    }
    
    public void setSfxEvent(long startTime, Procedural sfxPar, float minRandPeriodPar, float maxRandPeriodPar) 
    {
        eventTime = Common.currentFrameStartTimeMs + startTime;
        sfx = sfxPar;
        
        if(minRandPeriodPar>0)
        {
            type = EVENT_RANDOM_REPEATABLE_SFX;
            minRandPeriod = minRandPeriodPar;
            maxRandPeriod = maxRandPeriodPar;
        }
        else
        {
            type = EVENT_SFX;
        }
    }
    
    public void setSpiralAnimationEffect(long startTime, Sprite zombie, float x, float y)
    {
        clear();
        eventTime = Common.currentFrameStartTimeMs + startTime;
        type = EVENT_START_ZOMBIE_SPIRAL;
        sprite = zombie;
        posX = x;
        posY = y;
        System.out.println("setSpiralAnimationEffect((): currentFrameStartTimeMs=" + Common.currentFrameStartTimeMs);
        System.out.println("setSpiralAnimationEffect((): eventTime=" + eventTime);
    }
    
    
    public void execute()
    {
        if( type == EVENT_START_BUBBLE )
        {
            eventTime = Common.currentFrameStartTimeMs + (long)(2*1000);
            type = EVENT_END_BUBBLE;
            isDrawingState = true; 
        }
        else if( type == EVENT_END_BUBBLE )
        {
            clear();    
        }
        else if( type == EVENT_SFX )
        {
            sfx.play();
            clear();      
        }
        else if( type == EVENT_RANDOM_REPEATABLE_SFX )
        {
            // Play sound.
            //System.out.println("Event.execute(). play sfx");
            sfx.play();
            
            // Rattle a new sfx time, and relaunch the event.
            float startTime = ( Math.random() * (maxRandPeriod-minRandPeriod) ) + minRandPeriod;
            setSfxEvent((int)startTime, sfx, minRandPeriod, maxRandPeriod);
        }
        else if( type == EVENT_START_ZOMBIE_SPIRAL )
        {
            eventTime = Common.currentFrameStartTimeMs + (long)(1000*1000); // no timed stop
            type = EVENT_END_ZOMBIE_SPIRAL;
            isDrawingState = true; 
            phase = 0;
            phaseEndTime = Common.currentFrameStartTimeMs + (long)(500);
            System.out.println("execute(): currentFrameStartTimeMs=" + Common.currentFrameStartTimeMs);
            System.out.println("execute(): phaseEndTime=" + phaseEndTime);
        }
    }
    
    public void draw(HiRes16Color screen)
    {
        if( type == EVENT_END_BUBBLE )
        {
            image.draw(screen, 100, 60);
            screen.setTextPosition( 100+5, 60+5 );
            screen.textColor = 0;
            screen.print(textLine1);
            screen.setTextPosition( 100+5, 60+5+8 );
            screen.print(textLine2);
         }
        else if( type == EVENT_END_ZOMBIE_SPIRAL )
        {
            if(phaseEndTime<Common.currentFrameStartTimeMs)
            {
                phase++;
                phaseEndTime = Common.currentFrameStartTimeMs + (long)(500);
                //!!HV TEST
                System.out.println("phase++ ***********");
                if(phase>3)
                {
                    clear(); // end event
                    System.out.println("end ***********");
                }
            }
                
            // !!HV TEST    
            // sprite.setMirrored(false);
            // sprite.setFlipped(false);
            // sprite.draw(screen, posX, posY);

            if( phase == 0)
            {
                sprite.setMirrored(false);
                sprite.setFlipped(false);
                sprite.draw(screen, posX, posY);
            }
            else if( phase == 1)
            {
                sprite.setMirrored(true);
                sprite.setFlipped(false);
                sprite.draw(screen, posX, posY);
            }
            else if( phase == 2)
            {
                sprite.setMirrored(false);
                sprite.setFlipped(true);
                sprite.draw(screen, posX, posY);
            }
            else if( phase == 3)
            {
                sprite.setMirrored(true);
                sprite.setFlipped(false);
                sprite.draw(screen, posX, posY);
            }
         }
    }
    
    public long eventTime;
    public int type;
    public boolean isDrawingState;
    //public EventState state;
    Image image;
    Sprite sprite;
    float posX;
    float posY;
    long phaseEndTime;
    int phase;
    
    // Bubble text event specific parameters
    public String textLine1;
    public String textLine2;
    
    // Sfx event specific parameters
    Procedural sfx;
    float minRandPeriod; 
    float maxRandPeriod;
}

